#include "vector.h"

// TODO Don't exit if realloc fail !
void reserve_vector(vector* v)
{
  element* test_data = realloc( v->data, v->size_used * REALLOCATION_COEFFICIANT * sizeof(element) );
  if(test_data == NULL ) {perror("realloc"); destroy_vector(v); exit(EXIT_FAILURE); }    
  v->data = test_data;
  v->capacity *= 2; 
}					


vector* create_vector(void (*init_element)(element*, const element), void (*destroy_element)(element))
{
  vector* v = malloc(sizeof(vector)); 
  if(v == NULL) 
    return NULL;

  v->data = malloc(DEFAULT_ALLOCATION_SIZE * sizeof(element));	
  if(v->data == NULL) 
  {
    free(v);
    return NULL;
  }
  v->capacity = DEFAULT_ALLOCATION_SIZE;
  v->size_used = 0;
  v->init_element = init_element;
  v->destroy_element = destroy_element;

  v->clone = clone_vector;
  v->assign = assign_vector;
  v->clear = clear_vector;
  v->destroy = destroy_vector;
  v->push_back = push_back_vector;
  v->pop_back = pop_back_vector;
  v->insert = insert_vector;
  v->size = size_vector;
  v->at = at_vector;
  v->front = front_vector;
  v->back = back_vector;
  v->empty = empty_vector;

  return v;
}

vector* clone_vector(vector const * const src)
{
  vector* cpy = create_vector(src->init_element, src->destroy_element);
  if(cpy == NULL)
    return NULL;
  src->assign(cpy, src);
  return cpy;
}

void destroy_vector(vector *v)
{
  v->clear(v);
  free(v->data);
  v->capacity = 0;
}

void assign_vector(vector *dst, vector const * const src)
{
  for(int i=0; i != src->size_used; ++i)
    dst->push_back(dst, src->data[i]); 
}

size_t size_vector(vector const * const v)
{
  return v->size_used;
}

element at_vector(vector const * const v, size_t n)
{
  return v->data[n-1];
}

element front_vector(vector const * const v)
{
  return v->data[0];
}

element back_vector(vector const * const v)
{
  return v->data[v->size_used - 1];
}

void push_back_vector(vector* v, element e)
{
  if(v->size_used == v->capacity)
    reserve_vector(v);
  v->init_element(&v->data[v->size_used++] , e);
}

void pop_back_vector(vector *v)
{
  v->destroy_element(v->data[--(v->size_used)]);
}

void modify_at_from_vector(vector *v, const element e, size_t n)
{
  v->destroy_element(v->data[n - 1]);
  v->init_element(&v->data[n-1], e);
}

void insert_vector(vector* v, const element e, size_t n)
{
  if(v->size_used == v->capacity)
    reserve_vector(v);
  int i;
  for(i = v->size_used; i != n-1; --i)
    v->data[i] = v->data[i-1];
  v->init_element(&v->data[i] , e);
  ++(v->size_used);
}

void clear_vector(vector *v)
{  
  for(; v->size_used != 0; --(v->size_used)) 
    v->destroy_element(v->data[v->size_used - 1]);
}
 
int empty_vector(vector const * const v)
{
  return v->size_used == 0;
}

