#ifndef VECTOR_H
#define VECTOR_H

#include "element.h"
#include <stdio.h>
#include <stdlib.h>

#define DEFAULT_ALLOCATION_SIZE 10
#define REALLOCATION_COEFFICIANT 2

typedef struct vector
{
  struct vector* (*clone)(struct vector const * const self);
  void (*assign)(struct vector *dst, struct vector const * const src);
  void (*clear)(struct vector *self);
  void (*destroy)(struct vector *self);
  void (*push_back)(struct vector *self, const element x);
  void (*pop_back)(struct vector *self);
  void (*insert)(struct vector *self, const element e, size_t n);
  size_t (*size)(struct vector const * const self);
  element (*at)(struct vector const * const self, size_t n);
  element (*front)(struct vector const * const self);
  element (*back)(struct vector const * const self);
  int (*empty)(struct vector const * const self);

  void (*init_element)(element* dst, const element src);
  void (*destroy_element)(element x);

  size_t capacity;
  size_t size_used;
  element *data;

} vector;

void reserve_vector(vector* v);
vector* create_vector(void (*init_element)(element*, const element), void (*destroy_element)(element));
vector* clone_vector(vector const * const src);
void destroy_vector(vector *v);
void assign_vector(vector *dst, vector const * const src);
size_t size_vector(vector const * const v);
element at_vector(vector const * const v, size_t n);
element front_vector(vector const * const v);
element back_vector(vector const * const v);
void push_back_vector(vector *v, const element e);
void pop_back_vector(vector *v);
void modify_at_from_vector(vector *v, const element e, size_t n);
void insert_vector(vector* v, const element e, size_t n);
void clear_vector(vector *v);
int empty_vector(vector const * const v);


#endif
